////////////////////////////////////////////////////////////////////////////////
// 
//  Copyright (c) 2010 Renaun Erickson <renaun.com>
// 
//  Permission is hereby granted, free of charge, to any person
//  obtaining a copy of this software and associated documentation
//  files (the "Software"), to deal in the Software without
//  restriction, including without limitation the rights to use,
//  copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following
//  conditions:
// 
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
// 
////////////////////////////////////////////////////////////////////////////////
/**
 Flex SDK Coding Conventions Used:
 http://opensource.adobe.com/wiki/display/flexsdk/Coding+Conventions
 **/
package com.renaun.net
{
	/**
	 * 	http://www.networksorcery.com/enp/rfc/rfc1035.txt
	 * 
	 * 	4.1.3. Resource record format
	 * 
	 *  Resource Record Types:
	 * 
	 *  A               1 a host address
	 *  NS              2 an authoritative name server
	 *  CNAME           5 the canonical name for an alias
	 *  SOA             6 marks the start of a zone of authority
	 *  WKS             11 a well known service description
	 *  PTR             12 a domain name pointer
	 *  HINFO           13 host information
	 *  MINFO           14 mailbox or mail list information
	 *  MX              15 mail exchange
	 *  TXT             16 text strings
	 *  SRV			 	33 Service record
	 */
	public class DNSResourceRecord
	{
		//--------------------------------------------------------------------------
		//
		//  Static Properties
		//
		//--------------------------------------------------------------------------
		
		public static const TYPE_A:String = "A";
		
		public static const TYPE_CNAME:String = "CNAME";
		public static const TYPE_PTR:String = "PTR";
		public static const TYPE_SRV:String = "SRV";
		public static const TYPE_TXT:String = "TXT";
		// TEMP for ones we don't parse right now
		public static const TYPE_UNSUPPORTED:String = "unsupported";
		
		//--------------------------------------------------------------------------
		//
		//  Constructor
		//
		//--------------------------------------------------------------------------
		
		public function DNSResourceRecord()
		{
		}
		
		//--------------------------------------------------------------------------
		//
		//  Variables
		//
		//--------------------------------------------------------------------------
		
		//----------------------------------
		//  subsection
		//----------------------------------
		
		//--------------------------------------------------------------------------
		//
		//  Properties
		//
		//--------------------------------------------------------------------------
		
		public var type:String = "";

		public var name:String = "";

		public var classType:int = 0;
		
		public var ttl:int = 0;
		
		public var data:*;
		
		//--------------------------------------------------------------------------
		//
		//  Methods
		//
		//--------------------------------------------------------------------------
		
	}
}