////////////////////////////////////////////////////////////////////////////////
// 
//  Copyright (c) 2010 Renaun Erickson <renaun.com>
// 
//  Permission is hereby granted, free of charge, to any person
//  obtaining a copy of this software and associated documentation
//  files (the "Software"), to deal in the Software without
//  restriction, including without limitation the rights to use,
//  copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following
//  conditions:
// 
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
// 
////////////////////////////////////////////////////////////////////////////////
/**
 Flex SDK Coding Conventions Used:
 http://opensource.adobe.com/wiki/display/flexsdk/Coding+Conventions
 **/
package com.renaun.net
{
	import com.renaun.messages.AccelerometerMessage;
	import com.renaun.messages.MessageFactory;
	import com.renaun.messages.TouchMessage;
	
	import flash.events.DatagramSocketDataEvent;
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.ServerSocketConnectEvent;
	import flash.net.DatagramSocket;
	import flash.net.ServerSocket;
	import flash.net.Socket;
	import flash.utils.ByteArray;
	import flash.utils.IDataInput;
	import flash.utils.IDataOutput;
	
	import mx.utils.ObjectUtil;
	
	import org.osmf.utils.BinarySearch;

	public class InputReceiver
	{
		//--------------------------------------------------------------------------
		//
		//  Constructor
		//
		//--------------------------------------------------------------------------
		
		public function InputReceiver()
		{
		}
		
		//--------------------------------------------------------------------------
		//
		//  Variables
		//
		//--------------------------------------------------------------------------
		
		private var udpSocket:DatagramSocket;
		private var tcpServerSocket:ServerSocket;
		private var serverListenSocket:Socket;
		private var tcpSocket:Socket;
		private var useTCP:Boolean = false;
		
		private var servicePort:int = 0;
		private var serviceAddress:String = "";
		
		
		//* Standard mDNS port and addresses for Bounjour
		private var mDNSPort:int = 5353;
		private var mDNSAddress:String = "224.0.0.251";
		private var dataMessage:DNSMessage;
		
		//----------------------------------
		//  subsection
		//----------------------------------
		
		//--------------------------------------------------------------------------
		//
		//  Properties
		//
		//--------------------------------------------------------------------------

		public var log:Function;
		
		public var multiTouch:Function;
		public var accelerometer:Function;
		public var requestHostPort:Function;
		
		public var serviceType:String = "_ree._udp";
		public var serviceName:String = "RenaunService";
		public var serviceDomain:String = "renaunmbp";
		
		[Bindable]
		public var connected:Boolean = false;
		
		//--------------------------------------------------------------------------
		//
		//  Methods
		//
		//--------------------------------------------------------------------------
		
		public function init(bothServer:Boolean = false):void
		{
			dataMessage = new DNSMessage();
			log("DatagramSocket Support: " + DatagramSocket.isSupported);
			if (DatagramSocket.isSupported)
			{
				try
				{
					udpSocket = new DatagramSocket();
					udpSocket.addEventListener(DatagramSocketDataEvent.DATA, dataHandler);
					udpSocket.addEventListener(IOErrorEvent.IO_ERROR, errorHandler);
					
				}
				catch (error:Error)
				{
					log("Error: " + error.errorID + ": " + error.message);
					useTCP = true;
				}
			}
			if (useTCP || !DatagramSocket.isSupported)
			{
				tcpSocket = new Socket();
				useTCP = true;
				requestHostPort();
			} 
		}
		
		public function connect(host:String, port:int):void
		{
			log("socket connect: " + host + ":" + port);
			try
			{
				tcpSocket.connect(host, port);
				//tcpSocket.addEventListener(Event.CONNECT, connectHandler);
				tcpSocket.addEventListener(ProgressEvent.SOCKET_DATA, socketDataHandler);
				connected = true;
			}
			catch (error:Error)
			{
				log("Socket ConnectError: " + error.errorID + ": " + error.message);
			}
		}
		
		private function processAction(obj:Object):void
		{
			if (obj.type == "bg")
			{
				
			}
			if (obj.type == Kai3Message.COMMAND_MULTITOUCH
				|| obj.type == Kai3Message.COMMAND_START_MULTITOUCH
				|| obj.type == Kai3Message.COMMAND_STOP_MULTITOUCH)
			{
				var touchMsg:TouchMessage;
				if (obj.type == Kai3Message.COMMAND_MULTITOUCH)
				{
					touchMsg = MessageFactory.createTouch(obj.data);
					obj.data = touchMsg
					log("TouchEvent: " + ObjectUtil.toString(touchMsg));
				}
				multiTouch(obj.type, obj.data);
			}
			
			if (obj.type == Kai3Message.COMMAND_ACCELEROMETER
				|| obj.type == Kai3Message.COMMAND_START_ACCELEROMETER
				|| obj.type == Kai3Message.COMMAND_STOP_ACCELEROMETER)
			{
				var dataMsg:AccelerometerMessage;
				if (obj.type == Kai3Message.COMMAND_ACCELEROMETER)
				{
					dataMsg = MessageFactory.createAccelerometer(obj.data);
					obj.data = dataMsg;
					log("AccelerometerMessage: " + ObjectUtil.toString(dataMsg));
				}
				accelerometer(obj.type, obj.data);
			}
		}
		
		/**
		 * 	This is the sending Sockets listener
		 */
		private function socketDataHandler(event:ProgressEvent):void
		{
			log("socket data");
		}
		
		/**
		 * 	Each response could be quite different but you loop
		 *  through the DNS answers and additional records to figure
		 *  out what data you want to use.
		 */
		private function dataHandler(event:DatagramSocketDataEvent):void
		{
			processBytes(event.data);
		}
		
		public function processBytes(dataBytes:ByteArray):void
		{
			//log("DATA["+event.srcAddress+":"+event.srcPort+"]: " + event.data.length);
			//log("DATA: " + event.data.length);
			if (dataBytes.readUTFBytes(3) == "amf")
			{
				// Whats the AMF message
				var obj:Object = dataBytes.readObject();
				processAction(obj);
			}
			else
			{
				dataBytes.position = 0;
				try
				{
					dataMessage.parse(dataBytes);
				}
				catch (error:Error)
				{
					return;
				}
				var answersLength:int = dataMessage.answers.length;
				if (answersLength == 0)
				{
					dataBytes.position = 0;
					
					log("Message From: " + dataBytes.readUTFBytes(dataBytes.length));
					
					return;
				}
				log("Answer Resources: " + answersLength);
				for (var i:int = 0; i < answersLength; i++)
				{
					if (dataMessage.answers[i].type == DNSResourceRecord.TYPE_PTR)
					{
						log("Answer Data: " + dataMessage.answers[i].data);
						var additionalLength:int = dataMessage.additionals.length;
						for (var j:int = 0; j < additionalLength; j++)
						{
							if (dataMessage.additionals[j].type == DNSResourceRecord.TYPE_SRV)
							{	
								log("Answer Port: " + dataMessage.additionals[j].data.port);
								servicePort = int(dataMessage.additionals[j].data.port);
							}
							if (dataMessage.additionals[j].type == DNSResourceRecord.TYPE_A)
							{	
								log("Answer Address: " + dataMessage.additionals[j].data);
								serviceAddress = dataMessage.additionals[j].data;
								connected = true;
							}
						}
					}
				}
			}
		}
		
		private function errorHandler(event:ErrorEvent):void
		{
			log("ERROR: " + event.text + "\n");
		}
		
		
		
		/**
		 * 	Send standard DNS requests.
		 *  http://www.ietf.org/rfc/rfc1035.txt
		 */
		public function findService():void
		{
			log("sendQuery");
			bindSetup(-1);				
			
			log("packing bytes");
			var bytes:ByteArray = new ByteArray();
			
			// DNS Header
			bytes.writeFloat(0); 
			bytes.writeShort(1);	// 1 Question
			bytes.writeShort(0); // No Answer RRs
			bytes.writeShort(0); // No Authority RRs
			bytes.writeShort(0); // No Additional RRs
			// Query Name Chunk
			writeQueryName(bytes, serviceType);
			
			// Write out Type and Class (PTR/12) and 00 01
			bytes.writeByte(0); // 00
			bytes.writeByte(12); // 0C
			bytes.writeByte(0); // 00
			bytes.writeByte(1); // 01
			
			// Send DNS packet to mDNS port and address, since we are sending from a 
			// non-5353 port it will send the response back as a unicast DNS answer
			udpSocket.send(bytes, 0, 0, mDNSAddress, mDNSPort);
		}
		
		
		
		public function bindSetup(port:int):void
		{
			// Don't bind until first message sent out and only bind once
			if (udpSocket.localAddress == null)
			{	
				log("no localAddress");
				try
				{
					log("binding: " + port);
					if (port > 0)
					{
						udpSocket.bind(port);
						
						tcpServerSocket = new ServerSocket();
						tcpServerSocket.bind((port+1));
						tcpServerSocket.addEventListener(ServerSocketConnectEvent.CONNECT, socketConnectHandler);
						tcpServerSocket.listen();
						log("tcp Binding: " + tcpServerSocket.localPort);
					}
					else
					{
						udpSocket.bind();
					}
					log("receive");
					udpSocket.receive();			
					log("Receiving on: " + udpSocket.localAddress+":"+udpSocket.localPort);
					serviceDomain = udpSocket.localAddress;
					
				}
				catch (error:Error)
				{
					log("Error: " + error.errorID + ": " + error.message);
				}
			}
		}
		private function socketConnectHandler(event:ServerSocketConnectEvent):void
		{
			log("SocketServer Connected");
			var socket:Socket = event.socket;
			socket.addEventListener(ProgressEvent.SOCKET_DATA, socketListenDataHandler);
		}
		
		private function socketListenDataHandler(event:ProgressEvent):void
		{
			log("data");
			try
			{
				var socket:Socket = event.target as Socket;
				socket.removeEventListener(ProgressEvent.SOCKET_DATA, socketDataHandler);
				var bytes:ByteArray = new ByteArray();
				socket.readBytes(bytes);
				processBytes(bytes);
			}
			catch (error:Error)
			{
				log("ServerSocket Listen: Error" + error.message);
			}
		}

		
		/**
		 * 	Splits the service mDNS string and packs the bytes 
		 *  correctly in the <code>bytes</codes> parameter.
		 */
		private function writeQueryName(bytes:ByteArray, query:String = "_ree._udp"):void
		{
			var arr:Array = query.split(".");
			var len:int = arr.length;
			for (var i:int = 0; i < len; i++)
			{
				bytes.writeByte((arr[i] as String).length);
				bytes.writeUTFBytes(arr[i]);
			}
			// Add .local and 00
			bytes.writeByte(5);
			bytes.writeUTFBytes("local");
			bytes.writeByte(0);
		}
		
		
		
		/**
		 * 	Send standard DNS requests.
		 *  http://www.ietf.org/rfc/rfc1035.txt
		 */
		public function sendData(obj:Kai3Message):void
		{
			if (useTCP)
			{
				tcpSocket.writeUTFBytes("amf");
				tcpSocket.writeObject(obj);
				tcpSocket.flush();
			}
			else
			{
				//log("sendData to " + serviceAddress + ":" + servicePort);
				//bindSetup();
				var bytes:ByteArray = new ByteArray();
				bytes.writeUTFBytes("amf");
				bytes.writeObject(obj);
				
				udpSocket.send(bytes, 0, 0, serviceAddress, servicePort);
			}
		}
	}
}