////////////////////////////////////////////////////////////////////////////////
// 
//  Copyright (c) 2010 Renaun Erickson <renaun.com>
// 
//  Permission is hereby granted, free of charge, to any person
//  obtaining a copy of this software and associated documentation
//  files (the "Software"), to deal in the Software without
//  restriction, including without limitation the rights to use,
//  copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following
//  conditions:
// 
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
// 
////////////////////////////////////////////////////////////////////////////////
/**
 Flex SDK Coding Conventions Used:
 http://opensource.adobe.com/wiki/display/flexsdk/Coding+Conventions
 **/
package com.renaun.windows
{
	import com.renaun.messages.TouchMessage;
	
	import flash.display.NativeWindow;
	import flash.display.NativeWindowInitOptions;
	import flash.display.Shape;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.TouchEvent;
	import flash.utils.Dictionary;
	
	import org.osmf.display.ScaleMode;
	
	public class DrawWindow extends NativeWindow
	{
		//--------------------------------------------------------------------------
		//
		//  Constructor
		//
		//--------------------------------------------------------------------------
		
		public function DrawWindow(initOptions:NativeWindowInitOptions)
		{
			super(initOptions);
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
		}
		
		//--------------------------------------------------------------------------
		//
		//  Variables
		//
		//--------------------------------------------------------------------------
		
		
		//----------------------------------
		//  subsection
		//----------------------------------
		
		//--------------------------------------------------------------------------
		//
		//  Properties
		//
		//--------------------------------------------------------------------------
		
		public var lines:Dictionary = new Dictionary(true);
		public var oldLines:Vector.<Shape> = new Vector.<Shape>();
		private var bg:Shape;
		
		//--------------------------------------------------------------------------
		//
		//  Methods
		//
		//--------------------------------------------------------------------------
		
		public function processMessage(message:TouchMessage):void
		{
			//trace("process mesage: " + oldLines.length);
			var shape:Shape = lines[message.touchPointID];
			if (message.type == TouchEvent.TOUCH_BEGIN)
			{
				
				//trace("process mesage BEGIN");
				shape = new Shape();
				var color:uint = (Math.random() * 0xdd) + ((Math.random() * 0xdd)  << 8) + ((Math.random() * 0xdd)  << 16);
				
				shape.graphics.lineStyle(8, color, 0.8, true);
				shape.graphics.moveTo(message.localX, message.localY);
				lines[message.touchPointID] = shape;
				this.stage.addChild(shape);
				
				if (oldLines.length > 10)
				{
					shape = oldLines.shift();
					if (shape)
						stage.removeChild(shape);
					shape = null;
				}
			}
			else if (message.type == TouchEvent.TOUCH_MOVE)
			{
				
				//trace("process mesage MOVE");
				if (shape)
					shape.graphics.lineTo(message.localX, message.localY);
			}
			else if (message.type == TouchEvent.TOUCH_END)
			{
				
				//trace("process mesage END");
				oldLines.push(shape);
				lines[message.touchPointID] = null;
				delete lines[message.touchPointID];
				
			}
		}
	}
}